use std::path::Path;

use serde::Deserialize;
use url::Url;

use crate::getter::DanbooruPost;

pub fn ask_confirm(question: &str) -> bool {
    println!("{}", question);
    loop {
        let mut input = String::new();
        let _ = std::io::stdin().read_line(&mut input);
        match input.trim().chars().next() {
            Some('y') | Some('Y') => return true,
            Some('n') | Some('N') => return false,
            _ => println!("y/n only please."),
        }
    }
}

pub fn get_extension(file_url: &str) -> Result<String, ()> {
    match Url::parse(file_url)
        .ok()
        .as_mut()
        .and_then(|url| url.path_segments())
        .and_then(|path| path.last().map(|file_name| file_name.to_string()))
        .and_then(|file_name| file_name.split('.').last().map(|ext| ext.to_string()))
    {
        Some(extension) => Ok(extension),
        None => Err(()),
    }
}

#[derive(Deserialize, Debug)]
pub struct FilteredDanbooruPost {
    pub id: u64,
    pub file_url: String,
    pub file_size: u64,
}

pub fn filter_posts(path: &str, posts: Vec<DanbooruPost>) -> (Vec<FilteredDanbooruPost>, u64, u64) {
    let mut downloaded = 0;
    let mut failed = 0;

    let filtered_posts: Vec<FilteredDanbooruPost> = posts
        .into_iter()
        .filter_map(|p| {
            if let (Some(file_url), Some(file_size)) = (p.large_file_url, p.file_size) {
                Some(FilteredDanbooruPost {
                    id: p.id,
                    file_url,
                    file_size,
                })
            } else {
                failed += 1;
                None
            }
        })
        .filter(|post| match get_extension(post.file_url.as_str()) {
            Ok(extension) => {
                let filename = format!("{}/{}.{}", path, post.id, extension);
                if Path::new(&filename).exists() {
                    downloaded += 1;
                    false
                } else {
                    true
                }
            }
            Err(_) => {
                println!("Failed to get file extension for post: {}", post.id);
                false
            }
        })
        .collect();

    (filtered_posts, downloaded, failed)
}
